#![allow(dead_code)]
use ed25519_dalek::{SigningKey, VerifyingKey};
use once_cell::sync::Lazy;
use std::collections::HashMap;
use std::sync::Mutex;
use std::time::SystemTime;
use tabled::{Table, Tabled};

#[derive(Clone)]
pub struct NodeInfo {
    pub pubkey: VerifyingKey,
    pub updated_at: SystemTime,
}

static NODES: Lazy<Mutex<HashMap<String, NodeInfo>>> = Lazy::new(|| Mutex::new(HashMap::new()));

static KEYS: Lazy<Mutex<HashMap<VerifyingKey, SigningKey>>> =
    Lazy::new(|| Mutex::new(HashMap::new()));

pub fn add_key(pubkey: VerifyingKey, privkey: SigningKey) {
    let mut keys = KEYS.lock().unwrap();
    keys.insert(pubkey, privkey);
}

pub fn remove_key(pubkey: &VerifyingKey) {
    let mut keys = KEYS.lock().unwrap();
    keys.remove(pubkey);
}

pub fn get_privkey(pubkey: &VerifyingKey) -> Option<SigningKey> {
    let keys = KEYS.lock().unwrap();
    keys.get(pubkey).cloned()
}

pub fn add_node(ip: String, info: NodeInfo) {
    let mut nodes = NODES.lock().unwrap();
    nodes.insert(ip, info);
}

pub fn remove_node(ip: &str) {
    let mut nodes = NODES.lock().unwrap();
    nodes.remove(ip);
}

pub fn get_pubkey(ip: &str) -> Option<NodeInfo> {
    let nodes = NODES.lock().unwrap();
    nodes.get(ip).cloned()
}

pub fn get_nodes_as_html_tabe() -> String {
    #[derive(Tabled)]
    struct OutputRow {
        ip: String,
        pubkey: String,
        age: u64,
    }
    let mut output = vec![];
    for (ip, node_info) in NODES.lock().unwrap().iter() {
        let ip = ip.clone();
        let pubkey = hex::encode(node_info.pubkey.as_bytes());
        let age = std::time::SystemTime::now()
            .duration_since(node_info.updated_at)
            .unwrap_or(std::time::Duration::ZERO)
            .as_secs();
        output.push(OutputRow { ip, pubkey, age });
    }
    Table::new(output).to_string()
}
